@@ -152,7 +152,10 @@ _asn1_append_sequence_set (asn1_node node, struct node_tail_cache_st *pcache)
   while ((type_field (p->type) == ASN1_ETYPE_TAG)
 	 || (type_field (p->type) == ASN1_ETYPE_SIZE))
     p = p->right;
+
   p2 = _asn1_copy_structure3 (p);
+  if (p2 == NULL)
+    return ASN1_GENERIC_ERROR;
 
   if (pcache == NULL || pcache->tail == NULL || pcache->head != node)
     {
@@ -750,7 +753,8 @@ asn1_write_value (asn1_node node_root, const char *name,
  *   %ASN1_VALUE_NOT_FOUND if there isn't any value for the element
  *   selected, and %ASN1_MEM_ERROR if The value vector isn't big enough
  *   to store the result, and in this case @len will contain the number of
- *   bytes needed.
+ *   bytes needed. On the occasion that the stored data are of zero-length
+ *   this function may return %ASN1_SUCCESS even if the provided @len is zero.
  **/
 int
 asn1_read_value (asn1_node root, const char *name, void *ivalue, int *len)
@@ -823,7 +827,8 @@ asn1_read_value (asn1_node root, const char *name, void *ivalue, int *len)
  *   %ASN1_VALUE_NOT_FOUND if there isn't any value for the element
  *   selected, and %ASN1_MEM_ERROR if The value vector isn't big enough
  *   to store the result, and in this case @len will contain the number of
- *   bytes needed.
+ *   bytes needed. On the occasion that the stored data are of zero-length
+ *   this function may return %ASN1_SUCCESS even if the provided @len is zero.
  **/
 int
 asn1_read_value_type (asn1_node root, const char *name, void *ivalue,
@@ -929,7 +934,7 @@ asn1_read_value_type (asn1_node root, const char *name, void *ivalue,
 	{
 	  *len = 0;
 	  if (value)
-	  	value[0] = 0;
+	    value[0] = 0;
 	  p = node->down;
 	  while (p)
 	    {
